@@ -21,6 +21,11 @@
 #  define OPENLDAP_ASYNC_WORKAROUND
 #endif
 
+/* Solaris LDAP library doesn't have LDAP_OPT_SUCCESS */
+#ifndef LDAP_OPT_SUCCESS
+#  define LDAP_OPT_SUCCESS LDAP_SUCCESS
+#endif
+
 #define DEF(type, name) \
 	{ type, #name, offsetof(struct ldap_settings, name) }
 
@@ -112,6 +117,7 @@ void db_ldap_search(struct ldap_connection *conn, const char *base, int scope,
 	if (!conn->connected) {
 		if (!ldap_conn_open(conn)) {
 			request->callback(conn, request, NULL);
+			i_free(request);
 			return;
 		}
 	}
@@ -121,6 +127,7 @@ void db_ldap_search(struct ldap_connection *conn, const char *base, int scope,
 		i_error("LDAP: ldap_search() failed (filter %s): %s",
 			filter, ldap_get_error(conn));
 		request->callback(conn, request, NULL);
+		i_free(request);
 		return;
 	}
 
