@@ -14,8 +14,6 @@
 
 #include <stdlib.h>
 
-#define SERVICE_TYPE "imap"
-
 /* Linear whitespace */
 #define IS_LWS(c) ((c) == ' ' || (c) == '\t')
 
@@ -280,23 +278,6 @@ static int parse_next(char **data, char **key, char **value)
 	return TRUE;
 }
 
-/* remove leading and trailing whitespace */
-static const char *trim(const char *str)
-{
-	const char *ret;
-
-	while (IS_LWS(*str)) str++;
-	ret = str;
-
-	while (*str != '\0') str++;
-	if (str > ret) {
-		while (IS_LWS(str[-1])) str--;
-		ret = t_strdup_until(ret, str);
-	}
-
-	return ret;
-}
-
 static int auth_handle_response(struct digest_auth_request *auth,
 				char *key, char *value, const char **error)
 {
@@ -400,11 +381,6 @@ static int auth_handle_response(struct digest_auth_request *auth,
 			return FALSE;
 		}
 
-		if (strcasecmp(trim(uri[0]), SERVICE_TYPE) != 0) {
-			*error = "Unexpected service type in digest-uri";
-			return FALSE;
-		}
-
 		/* FIXME: RFC recommends that we verify the host/serv-type.
 		   But isn't the realm enough already? That'd be just extra
 		   configuration.. Maybe optionally list valid hosts in
