@@ -41,7 +41,7 @@ mech_plain_auth_continue(struct auth_request *auth_request,
 		}
 	}
 
-	if (authenid == NULL) {
+	if (count != 2) {
 		/* invalid input */
 		if (verbose)
 			i_info("mech-plain: no username given");
