@@ -41,7 +41,7 @@ static void passwd_file_init(const char *args)
 		userdb_pwf->refcount++;
 
 		/* resync */
-		userdb_pwf->userdb = FALSE;
+		userdb_pwf->userdb = TRUE;
                 userdb_pwf->stamp = 0;
 	} else {
 		userdb_pwf = db_passwd_file_parse(args, TRUE);
