@@ -19,7 +19,7 @@
 static enum auth_mech auth_mechs = 0;
 static char *auth_mechs_capability = NULL;
 
-const char *client_authenticate_get_capabilities(int tls)
+const char *client_authenticate_get_capabilities(int secured)
 {
 	string_t *str;
 	int i;
@@ -35,7 +35,7 @@ const char *client_authenticate_get_capabilities(int tls)
 	for (i = 0; i < AUTH_MECH_COUNT; i++) {
 		if ((auth_mechs & auth_mech_desc[i].mech) &&
 		    auth_mech_desc[i].name != NULL &&
-		    (tls || !auth_mech_desc[i].plaintext ||
+		    (secured || !auth_mech_desc[i].plaintext ||
 		     !disable_plaintext_auth)) {
 			str_append_c(str, ' ');
 			str_append(str, "AUTH=");
@@ -95,7 +95,8 @@ static void master_callback(struct client *_client, int success)
 	} else {
 		reason = t_strconcat("Internal login failure: ",
 				     client->common.virtual_user, NULL);
-		client_send_line(client, "* BYE Internal login failure.");
+		client_send_line(client, "* BYE Internal login failure. "
+				 "Error report written to server log.");
 	}
 
 	client_destroy(client, reason);
@@ -167,7 +168,7 @@ int cmd_login(struct imap_client *client, struct imap_arg *args)
 	user = IMAP_ARG_STR(&args[0]);
 	pass = IMAP_ARG_STR(&args[1]);
 
-	if (!client->tls && disable_plaintext_auth) {
+	if (!client->secured && disable_plaintext_auth) {
 		client_send_line(client,
 			"* BAD [ALERT] Plaintext authentication is disabled, "
 			"but your client sent password in plaintext anyway."
@@ -302,7 +303,7 @@ int cmd_authenticate(struct imap_client *client, struct imap_arg *args)
 		return TRUE;
 	}
 
-	if (!client->tls && mech->plaintext && disable_plaintext_auth) {
+	if (!client->secured && mech->plaintext && disable_plaintext_auth) {
 		client_send_tagline(client,
 				    "NO Plaintext authentication disabled.");
 		return TRUE;
