@@ -359,7 +359,8 @@ int cmd_fetch(struct client *client)
 				client_sync_without_expunges(client);
 		}
 
-		client_send_tagline(client, ret > 0 ? "OK Fetch completed." :
+		client_send_tagline(client, ret > 0 || client->cmd_uid ?
+			"OK Fetch completed." :
 			"NO Some of the requested messages no longer exist.");
 	} else {
 		client_send_storage_error(client);
