@@ -320,3 +320,12 @@ void client_save_custom_flags(struct mailbox_custom_flags *dest,
 	for (i = 0; i < custom_flags_count; i++)
 		dest->custom_flags[i] = p_strdup(dest->pool, custom_flags[i]);
 }
+
+int mailbox_name_equals(const char *box1, const char *box2)
+{
+	if (strcmp(box1, box2) == 0)
+		return TRUE;
+
+	return strcasecmp(box1, "INBOX") == 0 && strcasecmp(box2, "INBOX") == 0;
+}
+
