@@ -113,7 +113,8 @@ static int fetch_send_rfc822(struct imap_fetch_context *ctx, struct mail *mail)
 		return FALSE;
 
 	return message_send(ctx->output, stream, &body_size,
-			    0, (uoff_t)-1, NULL, !mail->has_no_nuls) >= 0;
+			    0, body_size.virtual_size, NULL,
+			    !mail->has_no_nuls) >= 0;
 }
 
 static int fetch_send_rfc822_header(struct imap_fetch_context *ctx,
@@ -136,7 +137,8 @@ static int fetch_send_rfc822_header(struct imap_fetch_context *ctx,
 		return FALSE;
 
 	return message_send(ctx->output, stream, &hdr_size,
-			    0, (uoff_t)-1, NULL, !mail->has_no_nuls) >= 0;
+			    0, hdr_size.virtual_size, NULL,
+			    !mail->has_no_nuls) >= 0;
 }
 
 static int fetch_send_rfc822_text(struct imap_fetch_context *ctx,
@@ -160,7 +162,8 @@ static int fetch_send_rfc822_text(struct imap_fetch_context *ctx,
 
 	i_stream_seek(stream, hdr_size.physical_size);
 	return message_send(ctx->output, stream, &body_size,
-			    0, (uoff_t)-1, NULL, !mail->has_no_nuls) >= 0;
+			    0, body_size.virtual_size, NULL,
+			    !mail->has_no_nuls) >= 0;
 }
 
 static int fetch_mail(struct imap_fetch_context *ctx, struct mail *mail)
