@@ -161,7 +161,8 @@ static int mmap_update(struct mail_index_data *data, uoff_t pos, size_t size)
 		if (pos + size <= data->mmap_used_length)
 			return TRUE;
 
-		if (pos + size <= data->mmap_full_length) {
+		if (pos + size <= data->mmap_full_length &&
+		    data->header != NULL) {
 			data->mmap_used_length = data->header->used_file_size;
 			if (data->mmap_used_length >=
 			    sizeof(struct mail_index_data_header) &&
