@@ -550,7 +550,7 @@ static int maildir_full_sync_finish(struct maildir_sync_context *ctx)
 		case MAILDIR_FILE_ACTION_UPDATE_FLAGS:
 			if (!maildir_update_filename(ctx, rec, orig_key))
 				return FALSE;
-			if (!maildir_update_flags(ctx, rec, seq, fname))
+			if (!maildir_update_flags(ctx, rec, seq, orig_key))
 				return FALSE;
 			break;
 		case MAILDIR_FILE_ACTION_UPDATE_CONTENT:
@@ -1040,6 +1040,8 @@ static int maildir_sync_new_dir(struct maildir_sync_context *ctx,
 		if (move_to_cur) {
 			str_truncate(destpath, 0);
 			str_printfa(destpath, "%s/%s", ctx->cur_dir, d->d_name);
+			if (strchr(d->d_name, ':') == NULL)
+				str_append(destpath, ":2,");
 
 			if (rename(str_c(sourcepath), str_c(destpath)) < 0 &&
 			    errno != ENOENT) {
@@ -1237,7 +1239,7 @@ static int maildir_full_sync_finish_readonly(struct maildir_sync_context *ctx)
 		    ACTION(hash_rec) == MAILDIR_FILE_ACTION_UPDATE_FLAGS) {
 			if (!maildir_update_filename(ctx, rec, orig_key))
 				return FALSE;
-			if (!maildir_update_flags(ctx, rec, seq, fname))
+			if (!maildir_update_flags(ctx, rec, seq, orig_key))
 				return FALSE;
 		}
 
@@ -1250,40 +1252,10 @@ static int maildir_full_sync_finish_readonly(struct maildir_sync_context *ctx)
 static int maildir_index_sync_context_readonly(struct maildir_sync_context *ctx)
 {
 	struct mail_index *index = ctx->index;
-	struct stat st;
-	int cur_changed;
 
 	i_assert(index->lock_type != MAIL_LOCK_UNLOCK);
 
-	if (stat(ctx->cur_dir, &st) < 0) {
-		index_file_set_syscall_error(index, ctx->cur_dir, "stat()");
-		return FALSE;
-	}
-
-	cur_changed = st.st_mtime != index->file_sync_stamp ||
-		index->maildir_cur_dirty != 0;
-
-	if (!cur_changed) {
-		if (!index->maildir_have_new) {
-			/* no changes */
-			return TRUE;
-		}
-
-		if (stat(ctx->new_dir, &st) < 0) {
-			return index_file_set_syscall_error(index, ctx->new_dir,
-							    "stat()");
-		}
-		if (st.st_mtime == index->last_new_mtime &&
-		    st.st_mtime < ioloop_time - MAILDIR_SYNC_SECS) {
-			/* no changes */
-			return TRUE;
-		}
-
-		if (!maildir_new_scan_first_file(ctx))
-			return FALSE;
-	}
-
-	/* ok, something's changed. check only changes in file names. */
+	/* check only changes in file names. */
 
 	/* if we can get exclusive lock, we can update the index
 	   directly. but don't rely on it. */
@@ -1343,6 +1315,10 @@ int maildir_index_sync_readonly(struct mail_index *index,
 	struct maildir_hash_rec *hash_rec;
 	int ret;
 
+	t_push();
+	/* fname may point into data file which syncing may break. */
+	fname = t_strdup(fname);
+
 	ctx = maildir_sync_context_new(index);
 	ctx->readonly_check = TRUE;
 
@@ -1356,6 +1332,7 @@ int maildir_index_sync_readonly(struct mail_index *index,
 			hash_rec->action != MAILDIR_FILE_ACTION_EXPUNGE;
 	}
 	maildir_index_sync_deinit(ctx);
+	t_pop();
 	return ret;
 }
 
