@@ -359,6 +359,11 @@ void mbox_header_cb(struct message_part *part __attr_unused__,
 					break;
 				ctx->uid = ctx->uid * 10 + (hdr->value[i]-'0');
 			}
+			if (ctx->uid >= ctx->index->header->next_uid) {
+				/* unexpectedly large UID field.
+				   don't allow it. */
+				ctx->uid = 0;
+			}
 		} else if (strcasecmp(hdr->name, "X-Status") == 0) {
 			/* update message flags */
 			ctx->flags |= mbox_get_status_flags(hdr->value,
