@@ -210,9 +210,9 @@ static int mbox_write_xstatus(struct mbox_rewrite_context *ctx,
 
 	str = t_strconcat("X-Status: ",
 			  (ctx->msg_flags & MAIL_ANSWERED) ? "A" : "",
-			  (ctx->msg_flags & MAIL_DRAFT) ? "D" : "",
+			  (ctx->msg_flags & MAIL_DELETED) ? "D" : "",
 			  (ctx->msg_flags & MAIL_FLAGGED) ? "F" : "",
-			  (ctx->msg_flags & MAIL_DELETED) ? "T" : "",
+			  (ctx->msg_flags & MAIL_DRAFT) ? "T" : "",
 			  x_status, NULL);
 
 	if (o_stream_send_str(ctx->output, str) < 0)
@@ -751,9 +751,11 @@ int mbox_index_rewrite(struct mail_index *index)
 	if (!failed)
 		reset_dirty_flags(index);
 
-	i_stream_unref(input);
 	o_stream_unref(output);
 
+	/* file may have grown, make sure v_size is updated */
+	mbox_file_close_stream(index);
+
 	if (!no_locking) {
 		if (!index->set_lock(index, MAIL_LOCK_UNLOCK))
 			failed = TRUE;
