@@ -132,7 +132,7 @@ static int mail_date_parse_tokens(struct message_tokenizer *ctx, time_t *time,
 
 	/* month name */
 	token = next_token(ctx, &value, &len);
-	if (token != 'A' || len != 3)
+	if (token != 'A' || len < 3)
 		return FALSE;
 
 	for (i = 0; i < 12; i++) {
@@ -198,17 +198,22 @@ static int mail_date_parse_tokens(struct message_tokenizer *ctx, time_t *time,
 		token = next_token(ctx, &value, &len);
 	}
 
-	/* timezone */
-	if (token != 'A')
-		return FALSE;
-	*timezone_offset = parse_timezone(value, len);
+	if (token == TOKEN_LAST) {
+		/* missing timezone */
+		*timezone_offset = 0;
+	} else {
+		/* timezone */
+		if (token != 'A')
+			return FALSE;
+		*timezone_offset = parse_timezone(value, len);
+	}
 
 	tm.tm_isdst = -1;
 	*time = utc_mktime(&tm);
 	if (*time == (time_t)-1)
 		return FALSE;
 
-	*time += *timezone_offset * 60;
+	*time -= *timezone_offset * 60;
 
 	return TRUE;
 }
