@@ -126,6 +126,14 @@ void settings_read(const char *path, settings_callback_t *callback,
 			*line++ = '\0';
 			while (IS_WHITE(*line)) line++;
 
+			len = strlen(line);
+			if (len > 0 &&
+			    ((*line == '"' && line[len-1] == '"') ||
+			     (*line == '\'' && line[len-1] == '\''))) {
+				line[len-1] = '\0';
+				line = str_unescape(line+1);
+			}
+
 			errormsg = callback(key, line, context);
 		}
 
