@@ -100,7 +100,7 @@ static int maildir_fill_readdir(struct mailbox_list_context *ctx,
 		/* check the type always since there's no extra cost */
 		if (d->d_type == DT_DIR)
 			;
-		else if (d->d_type != DT_UNKNOWN)
+		else if (d->d_type != DT_UNKNOWN && d->d_type != DT_LNK)
 			continue;
 		else
 #endif
