@@ -26,6 +26,8 @@ extern struct mailbox maildir_mailbox;
 
 static const char *maildirs[] = { "cur", "new", "tmp", NULL  };
 
+static int verify_inbox(struct mail_storage *storage);
+
 static struct mail_storage *maildir_create(const char *data, const char *user)
 {
 	struct mail_storage *storage;
@@ -82,11 +84,16 @@ static struct mail_storage *maildir_create(const char *data, const char *user)
 	storage->control_dir = i_strdup(control_dir);
 	storage->user = i_strdup(user);
 	storage->callbacks = i_new(struct mail_storage_callbacks, 1);
+	index_storage_init(storage);
+
+        (void)verify_inbox(storage);
 	return storage;
 }
 
 static void maildir_free(struct mail_storage *storage)
 {
+	index_storage_deinit(storage);
+
 	i_free(storage->dir);
 	i_free(storage->index_dir);
 	i_free(storage->control_dir);
@@ -301,14 +308,14 @@ maildir_open(struct mail_storage *storage, const char *name,
 	index_dir = maildir_get_index_path(storage, name);
 	control_dir = maildir_get_control_path(storage, name);
 
-	index = index_storage_lookup_ref(index_dir);
+	index = index_storage_lookup_ref(index_dir, path);
 	if (index == NULL) {
 		index = maildir_index_alloc(path, index_dir, control_dir);
 		index_storage_add(index);
 	}
 
-	ibox = index_storage_init(storage, &maildir_mailbox, index, name,
-				  readonly, fast);
+	ibox = index_storage_mailbox_init(storage, &maildir_mailbox,
+					  index, name, readonly, fast);
 	if (ibox != NULL)
 		ibox->expunge_locked = maildir_expunge_locked;
 	return (struct mailbox *) ibox;
@@ -500,14 +507,14 @@ static int rename_subfolders(struct mail_storage *storage,
         struct mailbox_list *list;
 	const char *oldpath, *newpath, *new_listname;
 	size_t oldnamelen;
-	int sorted, ret;
+	int ret;
 
 	ret = 0;
 	oldnamelen = strlen(oldname);
 
 	ctx = storage->list_mailbox_init(storage,
 					 t_strconcat(oldname, ".*", NULL),
-					 MAILBOX_LIST_FAST_FLAGS, &sorted);
+					 MAILBOX_LIST_FAST_FLAGS);
 	while ((list = maildir_list_mailbox_next(ctx)) != NULL) {
 		i_assert(oldnamelen <= strlen(list->name));
 
@@ -555,6 +562,7 @@ static int maildir_rename_mailbox(struct mail_storage *storage,
 	mail_storage_clear_error(storage);
 
 	oldname = inbox_fix_case(storage, oldname);
+	newname = inbox_fix_case(storage, newname);
 	if (!maildir_is_valid_existing_name(oldname) ||
 	    !maildir_is_valid_create_name(storage, newname)) {
 		mail_storage_set_error(storage, "Invalid mailbox name");
@@ -647,7 +655,7 @@ static int maildir_storage_close(struct mailbox *box)
 	}
 	ibox->index->set_lock_notify_callback(ibox->index, NULL, NULL);
 
-	return index_storage_close(box) && !failed;
+	return index_storage_mailbox_free(box) && !failed;
 }
 
 static void maildir_storage_auto_sync(struct mailbox *box,
