@@ -302,6 +302,10 @@ static int verify_inbox(struct mail_storage *storage)
 	fd = open(storage->inbox_file, O_RDWR | O_CREAT | O_EXCL, 0660);
 	if (fd != -1)
 		(void)close(fd);
+	else if (errno != EEXIST) {
+		mail_storage_set_critical(storage,
+			"open(%s, O_CREAT) failed: %m", storage->inbox_file);
+	}
 
 	/* make sure the index directories exist */
 	if (!create_mbox_index_dirs(storage, "INBOX"))
@@ -416,6 +420,17 @@ static int mbox_create_mailbox(struct mail_storage *storage, const char *name,
 		return FALSE;
 	}
 
+	if (strncasecmp(name, "INBOX/", 6) == 0) {
+		/* We might actually be able to create mailboxes under INBOX
+		   because the real INBOX file isn't usually named as INBOX
+		   in the root mail directory. that would anyway be a special
+		   case which would require special handling elsewhere, so just
+		   don't allow it. */
+		mail_storage_set_error(storage,
+				"Mailbox doesn't allow inferior mailboxes");
+		return -1;
+	}
+
 	/* make sure it doesn't exist already */
 	path = mbox_get_path(storage, name);
 	if (stat(path, &st) == 0) {
@@ -574,6 +589,13 @@ static int mbox_rename_mailbox(struct mail_storage *storage,
 		return FALSE;
 	}
 
+	if (strncasecmp(newname, "INBOX/", 6) == 0) {
+		/* Not allowed - see explanation in mbox_mailbox_create */
+		mail_storage_set_error(storage,
+			"Target mailbox doesn't allow inferior mailboxes");
+		return -1;
+	}
+
 	if (strcasecmp(oldname, "INBOX") == 0)
 		oldname = "INBOX";
 
