@@ -10,7 +10,7 @@
 
 /* Message to show to users when critical error occurs */
 #define CRITICAL_MSG \
-	"Internal error occured. Error report written to server log."
+	"Internal error occured. Refer to server log for more information."
 #define CRITICAL_MSG_STAMP CRITICAL_MSG " [%Y-%m-%d %H:%M:%S]"
 
 struct mail_storage_list {
@@ -26,6 +26,7 @@ struct client_workaround_list {
 struct client_workaround_list client_workaround_list[] = {
 	{ "oe6-fetch-no-newmail", WORKAROUND_OE6_FETCH_NO_NEWMAIL },
 	{ "outlook-idle", WORKAROUND_OUTLOOK_IDLE },
+	{ "outlook-pop3-no-nuls", WORKAROUND_OUTLOOK_POP3_NO_NULS },
 	{ NULL, 0 }
 };
 
