@@ -100,7 +100,8 @@ enum mail_sync_flags {
 
 enum client_workarounds {
 	WORKAROUND_OE6_FETCH_NO_NEWMAIL	= 0x01,
-	WORKAROUND_OUTLOOK_IDLE		= 0x02
+	WORKAROUND_OUTLOOK_IDLE		= 0x02,
+        WORKAROUND_OUTLOOK_POP3_NO_NULS	= 0x03
 };
 
 struct mail_full_flags {
