@@ -88,10 +88,13 @@ struct iovec {
 };
 #endif
 
-#ifndef HAVE_WRITEV
-#  define writev my_writev
-struct iovec;
-ssize_t my_writev(int fd, const struct iovec *iov, int iov_len);
+#if !defined (HAVE_PREAD) || defined (PREAD_WRAPPERS)
+#  ifndef IN_COMPAT_C
+#    define pread my_pread
+#    define pwrite my_pwrite
+#  endif
+ssize_t my_pread(int fd, void *buf, size_t count, off_t offset);
+ssize_t my_pwrite(int fd, const void *buf, size_t count, off_t offset);
 #endif
 
 /* ctype.h isn't safe with signed chars,
