@@ -1,42 +1,24 @@
+/* Copyright (c) 2002-2003 Timo Sirainen */
+
 /*
- fdpass.c - File descriptor passing between processes via UNIX sockets
+   fdpass.c - File descriptor passing between processes via UNIX sockets
 
- This isn't fully portable, but pretty much all UNIXes nowadays should
- support this. If you're having runtime problems, check the end of fd_read()
- and play with the if condition.
+   This isn't fully portable, but pretty much all UNIXes nowadays should
+   support this. If you're having runtime problems, check the end of fd_read()
+   and play with the if condition.
 
- If this file doesn't compile at all, you should check if this is supported
- in your system at all. It may require some extra #define to enable it.
- If not, you're pretty much out of luck. Cygwin didn't last I checked.
-
-    Copyright (c) 2002-2003 Timo Sirainen
-
-    Permission is hereby granted, free of charge, to any person obtaining
-    a copy of this software and associated documentation files (the
-    "Software"), to deal in the Software without restriction, including
-    without limitation the rights to use, copy, modify, merge, publish,
-    distribute, sublicense, and/or sell copies of the Software, and to
-    permit persons to whom the Software is furnished to do so, subject to
-    the following conditions:
-
-    The above copyright notice and this permission notice shall be
-    included in all copies or substantial portions of the Software.
-
-    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+   If this file doesn't compile at all, you should check if this is supported
+   in your system at all. It may require some extra #define to enable it.
+   If not, you're pretty much out of luck. Cygwin didn't last I checked.
 */
 
 #define _XPG4_2
 
 #if defined(irix) || defined (__irix__) || defined(sgi) || defined (__sgi__)
-#  define _XOPEN_SOURCE /* for IRIX */
+#  define _XOPEN_SOURCE 4 /* for IRIX */
 #endif
 
+
 #if !defined(_AIX) && !defined(_XOPEN_SOURCE_EXTENDED)
 #  define _XOPEN_SOURCE_EXTENDED /* for Tru64, breaks AIX */
 #endif
@@ -49,12 +31,19 @@
 #include <sys/uio.h>
 
 #ifndef CMSG_SPACE
-#  define CMSG_ALIGN(len) \
+#  if defined(_CMSG_DATA_ALIGN) && defined(_CMSG_HDR_ALIGN)  /* for Solaris */
+#    define CMSG_SPACE(len) \
+	(_CMSG_DATA_ALIGN(len) + _CMSG_HDR_ALIGN(sizeof(struct cmsghdr)))
+#    define CMSG_LEN(len) \
+	(_CMSG_DATA_ALIGN(sizeof(struct cmsghdr)) + (len))
+#  else
+#    define CMSG_ALIGN(len) \
 	(((len) + sizeof(size_t) - 1) & ~(sizeof(size_t) - 1))
-#  define CMSG_SPACE(len) \
+#    define CMSG_SPACE(len) \
 	(CMSG_ALIGN(len) + CMSG_ALIGN(sizeof(struct cmsghdr)))
-#  define CMSG_LEN(len) \
+#    define CMSG_LEN(len) \
 	(CMSG_ALIGN(sizeof(struct cmsghdr)) + (len))
+#  endif
 #endif
 
 #ifdef SCM_RIGHTS
@@ -79,6 +68,7 @@ ssize_t fd_send(int handle, int send_fd, const void *data, size_t size)
 
 	if (send_fd != -1) {
 		/* set the control and controllen before CMSG_FIRSTHDR() */
+		memset(buf, 0, sizeof(buf));
 		msg.msg_control = buf;
 		msg.msg_controllen = sizeof(buf);
 
@@ -96,6 +86,23 @@ ssize_t fd_send(int handle, int send_fd, const void *data, size_t size)
 	return sendmsg(handle, &msg, 0);
 }
 
+#ifdef __osf__
+#  define CHECK_MSG(msg) TRUE /* Tru64 */
+#else
+#  define CHECK_MSG(msg) (msg).msg_controllen >= CMSG_SPACE(sizeof(int))
+#endif
+
+#ifdef LINUX20
+/* Linux 2.0.x doesn't set any cmsg fields. Note that this might make some
+   attacks possible so don't do it unless you really have to. */
+#  define CHECK_CMSG(cmsg) ((cmsg) != NULL)
+#else
+#  define CHECK_CMSG(cmsg) \
+	((cmsg) != NULL && \
+	 (size_t)(cmsg)->cmsg_len >= (size_t)CMSG_LEN(sizeof(int)) && \
+	 (cmsg)->cmsg_level == SOL_SOCKET && (cmsg)->cmsg_type == SCM_RIGHTS)
+#endif
+
 ssize_t fd_read(int handle, void *data, size_t size, int *fd)
 {
 	struct msghdr msg;
@@ -127,17 +134,10 @@ ssize_t fd_read(int handle, void *data, size_t size, int *fd)
 	/* at least one byte transferred - we should have the fd now.
 	   do extra checks to make sure it really is an fd that is being
 	   transferred to avoid potential DoS conditions. some systems don't
-	   set all these values correctly however:
-
-	   Linux 2.0.x - cmsg_len, cmsg_level, cmsg_type are not set
-	   Tru64 - msg_controllen isn't set */
+	   set all these values correctly however so CHECK_MSG() and
+	   CHECK_CMSG() are somewhat system dependent */
 	cmsg = CMSG_FIRSTHDR(&msg);
-	if (
-#ifndef __osf__ /* Tru64 */
-	    msg.msg_controllen < CMSG_SPACE(sizeof(int)) ||
-#endif
-	    cmsg == NULL || cmsg->cmsg_len < CMSG_LEN(sizeof(int)) ||
-	    cmsg->cmsg_level != SOL_SOCKET || cmsg->cmsg_type != SCM_RIGHTS)
+	if (!CHECK_MSG(msg) || !CHECK_CMSG(cmsg))
 		*fd = -1;
 	else
 		*fd = *((int *) CMSG_DATA(cmsg));
