@@ -192,7 +192,16 @@ static ssize_t _read(struct _istream *stream)
 			return -1;
 		}
 
-		ret = read(stream->fd, stream->w_buffer + stream->pos, size);
+		if (fstream->file) {
+			ret = pread(stream->fd,
+				    stream->w_buffer + stream->pos, size,
+				    stream->istream.start_offset +
+				    stream->istream.v_offset +
+				    (stream->pos - stream->skip));
+		} else {
+			ret = read(stream->fd,
+				   stream->w_buffer + stream->pos, size);
+		}
 		if (ret == 0) {
 			/* EOF */
 			stream->istream.stream_errno = 0;
@@ -235,7 +244,10 @@ static void _skip(struct _istream *stream, uoff_t count)
 {
 	struct file_istream *fstream = (struct file_istream *) stream;
 
-	fstream->skip_left += count - (stream->pos - stream->skip);
+	if (!fstream->file)
+		fstream->skip_left += count - (stream->pos - stream->skip);
+
+	stream->istream.stream_errno = 0;
 	stream->skip = stream->pos = 0;
 	stream->istream.v_offset += count;
 }
@@ -243,32 +255,18 @@ static void _skip(struct _istream *stream, uoff_t count)
 static void _seek(struct _istream *stream, uoff_t v_offset)
 {
 	struct file_istream *fstream = (struct file_istream *) stream;
-	uoff_t real_offset;
-	off_t ret;
 
-	real_offset = stream->istream.start_offset + v_offset;
-	if (real_offset > OFF_T_MAX) {
-		stream->istream.stream_errno = EOVERFLOW;
-		ret = -1;
-	} else {
-		ret = lseek(stream->fd, (off_t)real_offset, SEEK_SET);
-		if (ret < 0)
-			stream->istream.stream_errno = errno;
-		else if (ret != (off_t)real_offset) {
-			stream->istream.stream_errno = EINVAL;
-			ret = -1;
-		} else {
-			stream->skip = stream->pos = 0;
-			fstream->skip_left = 0;
+	if (!fstream->file) {
+		if (v_offset < stream->istream.v_offset) {
+			stream->istream.stream_errno = ESPIPE;
+			return;
 		}
+		fstream->skip_left += v_offset - stream->istream.v_offset;
 	}
 
-	if (ret < 0)
-                i_stream_close(&stream->istream);
-	else {
-		stream->istream.stream_errno = 0;
-		stream->istream.v_offset = v_offset;
-	}
+	stream->istream.stream_errno = 0;
+	stream->istream.v_offset = v_offset;
+	stream->skip = stream->pos = 0;
 }
 
 struct istream *i_stream_create_file(int fd, pool_t pool,
