@@ -251,18 +251,21 @@ int i_stream_read_data(struct istream *stream, const unsigned char **data,
 {
 	struct _istream *_stream = stream->real_stream;
 	ssize_t ret = 0;
+	int read_bytes = FALSE;
 
 	while (_stream->pos - _stream->skip <= threshold) {
 		/* we need more data */
 		ret = i_stream_read(stream);
 		if (ret < 0)
 			break;
+		if (ret > 0)
+			read_bytes = TRUE;
 	}
 
 	*data = i_stream_get_data(stream, size);
 	return *size > threshold ? 1 :
 		ret == -2 ? -2 :
-		*size > 0 ? 0 : -1;
+		read_bytes ? 0 : -1;
 }
 
 struct istream *_i_stream_create(struct _istream *_stream, pool_t pool, int fd,
