@@ -1,27 +1,4 @@
-/*
-   network.c : Network stuff with IPv6 support
-
-    Copyright (c) 1999-2002 Timo Sirainen
-
-    Permission is hereby granted, free of charge, to any person obtaining
-    a copy of this software and associated documentation files (the
-    "Software"), to deal in the Software without restriction, including
-    without limitation the rights to use, copy, modify, merge, publish,
-    distribute, sublicense, and/or sell copies of the Software, and to
-    permit persons to whom the Software is furnished to do so, subject to
-    the following conditions:
-
-    The above copyright notice and this permission notice shall be
-    included in all copies or substantial portions of the Software.
-
-    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-*/
+/* Copyright (c) 1999-2003 Timo Sirainen */
 
 #include "lib.h"
 #include "network.h"
@@ -217,7 +194,18 @@ void net_disconnect(int fd)
 void net_set_nonblock(int fd __attr_unused__, int nonblock __attr_unused__)
 {
 #ifdef HAVE_FCNTL
-	if (fcntl(fd, F_SETFL, nonblock ? O_NONBLOCK : 0) < 0)
+	int flags;
+
+	flags = fcntl(fd, F_GETFL, 0);
+	if (flags == -1)
+		i_fatal("net_set_nonblock() failed: %m");
+
+	if (nonblock)
+		flags |= O_NONBLOCK;
+	else
+		flags &= ~O_NONBLOCK;
+
+	if (fcntl(fd, F_SETFL, flags) < 0)
 		i_fatal("net_set_nonblock() failed: %m");
 #endif
 }
@@ -454,7 +442,7 @@ int net_gethostbyname(const char *addr, struct ip_addr **ips, int *ips_count)
 	for (ai = origai; ai != NULL; ai = ai->ai_next, count++) {
 		so = (union sockaddr_union *) ai->ai_addr;
 
-		sin_get_ip(so, ips[count]);
+		sin_get_ip(so, &(*ips)[count]);
 	}
 	freeaddrinfo(origai);
 #else
@@ -499,6 +487,23 @@ int net_getsockname(int fd, struct ip_addr *addr, unsigned int *port)
 	return 0;
 }
 
+int net_getpeername(int fd, struct ip_addr *addr, unsigned int *port)
+{
+	union sockaddr_union so;
+	socklen_t addrlen;
+
+	i_assert(fd >= 0);
+
+	addrlen = sizeof(so);
+	if (getpeername(fd, (struct sockaddr *) &so, &addrlen) == -1)
+		return -1;
+
+        if (addr != NULL) sin_get_ip(&so, addr);
+	if (port != NULL) *port = sin_get_port(&so);
+
+	return 0;
+}
+
 const char *net_ip2addr(const struct ip_addr *ip)
 {
 #ifdef HAVE_IPV6
@@ -591,14 +596,18 @@ const char *net_gethosterror(int error)
 int net_hosterror_notfound(int error)
 {
 #ifdef HAVE_IPV6
+#ifdef EAI_NODATA /* NODATA is depricated */
 	return error != 1 && (error == EAI_NONAME || error == EAI_NODATA);
+#else
+	return error != 1 && (error == EAI_NONAME);
+#endif
 #else
 	return error == HOST_NOT_FOUND || error == NO_ADDRESS;
 #endif
 }
 
 /* Get name of TCP service */
-char *net_getservbyport(unsigned short port)
+const char *net_getservbyport(unsigned short port)
 {
 	struct servent *entry;
 
