@@ -95,8 +95,10 @@ static void _close(struct _iostream *stream)
 {
 	struct file_ostream *fstream = (struct file_ostream *) stream;
 
-	/* flush output before really closing it */
-	o_stream_flush(&fstream->ostream.ostream);
+	/* flush output before really closing it, unless we're here because
+	   we timeouted */
+	if (fstream->ostream.ostream.stream_errno != EAGAIN)
+		o_stream_flush(&fstream->ostream.ostream);
 
 	stream_closed(fstream);
 }
@@ -260,6 +262,7 @@ static int o_stream_send_blocking(struct file_ostream *fstream,
 	time_t timeout_time;
 	struct iovec iov[3];
 	int iov_len, first;
+	ssize_t ret;
 
 	iov_len = o_stream_fill_iovec(fstream, iov);
 	if (size > 0) {
@@ -276,14 +279,16 @@ static int o_stream_send_blocking(struct file_ostream *fstream,
 			first = FALSE;
 		else if (timeout_time > 0 && time(NULL) > timeout_time) {
 			/* timeouted */
+			fstream->ostream.ostream.stream_errno = EAGAIN;
 			if (fstream->timeout_cb != NULL)
 				fstream->timeout_cb(fstream->timeout_context);
-			fstream->ostream.ostream.stream_errno = EAGAIN;
 			return -1;
 		}
 
-		if (o_stream_writev(fstream, iov, iov_len) < 0)
+		if ((ret = o_stream_writev(fstream, iov, iov_len)) < 0)
 			return -1;
+		if (ret > 0)
+			timeout_time = GET_TIMEOUT_TIME(fstream);
 	}
 
         return 1;
@@ -462,7 +467,8 @@ static size_t o_stream_add(struct file_ostream *fstream,
 
 		if (unused > size-sent)
 			unused = size-sent;
-		memcpy(fstream->buffer + fstream->tail, data, unused);
+		memcpy(fstream->buffer + fstream->tail,
+		       (const char *)data + sent, unused);
 		sent += unused;
 
 		fstream->tail += unused;
@@ -536,7 +542,6 @@ static off_t io_stream_sendfile(struct _ostream *outstream,
 	int first;
 
 	/* set timeout time before hflushing existing buffer which may block */
-	timeout_time = GET_TIMEOUT_TIME(foutstream);
         start_offset = instream->v_offset;
 
 	/* flush out any data in buffer */
@@ -546,16 +551,17 @@ static off_t io_stream_sendfile(struct _ostream *outstream,
 	v_offset = instream->v_offset;
 
 	first = TRUE;
+	timeout_time = GET_TIMEOUT_TIME(foutstream);
 	do {
 		if (first)
 			first = FALSE;
 		else if (timeout_time > 0 && time(NULL) > timeout_time) {
 			/* timeouted */
+			outstream->ostream.stream_errno = EAGAIN;
 			if (foutstream->timeout_cb != NULL) {
 				foutstream->timeout_cb(
 					foutstream->timeout_context);
 			}
-			outstream->ostream.stream_errno = EAGAIN;
 			ret = -1;
 			break;
 		}
@@ -581,6 +587,8 @@ static off_t io_stream_sendfile(struct _ostream *outstream,
 				/* don't block */
 				break;
 			}
+		} else {
+			timeout_time = GET_TIMEOUT_TIME(foutstream);
 		}
 
 		v_offset += ret;
@@ -604,7 +612,6 @@ static off_t io_stream_copy(struct _ostream *outstream,
 	ssize_t ret;
 	int pos;
 
-	timeout_time = GET_TIMEOUT_TIME(foutstream);
 	iov_len = o_stream_fill_iovec(foutstream, iov);
 
         skip_size = 0;
@@ -613,6 +620,7 @@ static off_t io_stream_copy(struct _ostream *outstream,
 
 	i_assert(!overlapping || iov_len == 0);
 
+	timeout_time = GET_TIMEOUT_TIME(foutstream);
         start_offset = instream->v_offset;
 	for (;;) {
 		if (overlapping)
@@ -641,7 +649,9 @@ static off_t io_stream_copy(struct _ostream *outstream,
 			return -1;
 		}
 
-		if (ret == 0 && !STREAM_IS_BLOCKING(foutstream)) {
+		if (ret > 0)
+			timeout_time = GET_TIMEOUT_TIME(foutstream);
+		else if (!STREAM_IS_BLOCKING(foutstream)) {
 			/* don't block */
 			break;
 		}
@@ -659,11 +669,11 @@ static off_t io_stream_copy(struct _ostream *outstream,
 
 		if (timeout_time > 0 && time(NULL) > timeout_time) {
 			/* timeouted */
+			outstream->ostream.stream_errno = EAGAIN;
 			if (foutstream->timeout_cb != NULL) {
 				foutstream->timeout_cb(
 					foutstream->timeout_context);
 			}
-			outstream->ostream.stream_errno = EAGAIN;
 			return -1;
 		}
 
@@ -694,8 +704,6 @@ static off_t io_stream_copy_backwards(struct _ostream *outstream,
 
 	i_assert(IS_STREAM_EMPTY(foutstream));
 
-	timeout_time = GET_TIMEOUT_TIME(foutstream);
-
 	/* figure out optimal buffer size */
 	buffer_size = instream->real_stream->buffer_size;
 	if (buffer_size == 0 || buffer_size > foutstream->buffer_size) {
@@ -715,6 +723,7 @@ static off_t io_stream_copy_backwards(struct _ostream *outstream,
 
 	i_assert(out_offset <= instream->start_offset + instream->v_size);
 
+	timeout_time = GET_TIMEOUT_TIME(foutstream);
 	while (in_offset > in_start_offset) {
 		if (in_offset - in_start_offset <= buffer_size)
 			read_size = in_offset - in_start_offset;
@@ -763,14 +772,16 @@ static off_t io_stream_copy_backwards(struct _ostream *outstream,
 			outstream->ostream.stream_errno = errno;
 			return -1;
 		}
+		if (ret > 0)
+			timeout_time = GET_TIMEOUT_TIME(foutstream);
 
 		if (timeout_time > 0 && time(NULL) > timeout_time) {
 			/* timeouted */
+			outstream->ostream.stream_errno = EAGAIN;
 			if (foutstream->timeout_cb != NULL) {
 				foutstream->timeout_cb(
 					foutstream->timeout_context);
 			}
-			outstream->ostream.stream_errno = EAGAIN;
 			return -1;
 		}
 
