@@ -1,25 +1,4 @@
-/*
-    Copyright (c) 2002 Timo Sirainen
-
-    Permission is hereby granted, free of charge, to any person obtaining
-    a copy of this software and associated documentation files (the
-    "Software"), to deal in the Software without restriction, including
-    without limitation the rights to use, copy, modify, merge, publish,
-    distribute, sublicense, and/or sell copies of the Software, and to
-    permit persons to whom the Software is furnished to do so, subject to
-    the following conditions:
-
-    The above copyright notice and this permission notice shall be
-    included in all copies or substantial portions of the Software.
-
-    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-*/
+/* Copyright (c) 2002-2004 Timo Sirainen */
 
 #include "lib.h"
 #include "restrict-access.h"
@@ -30,12 +9,6 @@
 #include <time.h>
 #include <grp.h>
 
-#define HARD_MAX_GROUPS 10240
-
-#ifndef NGROUPS_MAX
-#  define NGROUPS_MAX 128
-#endif
-
 void restrict_access_set_env(const char *user, uid_t uid, gid_t gid,
 			     const char *chroot_dir,
 			     gid_t first_valid_gid, gid_t last_valid_gid,
@@ -65,21 +38,16 @@ void restrict_access_set_env(const char *user, uid_t uid, gid_t gid,
 
 static gid_t *get_groups_list(int *gid_count_r)
 {
-	/* @UNSAFE */
 	gid_t *gid_list;
 	int ret, gid_count;
 
-	gid_count = NGROUPS_MAX;
-	gid_list = t_buffer_get(sizeof(gid_t) * gid_count);
-	while ((ret = getgroups(gid_count, gid_list)) < 0) {
-		if (errno != EINVAL ||
-		    gid_count < HARD_MAX_GROUPS)
-			i_fatal("getgroups() failed: %m");
+	if ((gid_count = getgroups(0, NULL)) < 0)
+		i_fatal("getgroups() failed: %m");
 
-		gid_count *= 2;
-		gid_list = t_buffer_reget(gid_list, sizeof(gid_t) * gid_count);
-	}
-	t_buffer_alloc(sizeof(gid_t) * ret);
+	/* @UNSAFE */
+	gid_list = t_new(gid_t, gid_count);
+	if ((ret = getgroups(gid_count, gid_list)) < 0)
+		i_fatal("getgroups() failed: %m");
 
 	*gid_count_r = ret;
 	return gid_list;
@@ -216,13 +184,22 @@ void restrict_access_by_env(int disallow_root)
 
 	/* verify that we actually dropped the privileges */
 	if (uid != 0 || disallow_root) {
-		if (setuid(0) == 0)
+		if (setuid(0) == 0) {
+			if (uid == 0)
+				i_fatal("Running as root isn't permitted");
 			i_fatal("We couldn't drop root privileges");
+		}
 	}
 
-	if ((gid != 0 && uid != 0) || disallow_root) {
-		if (getgid() == 0 || getegid() == 0 || setgid(0) == 0)
-			i_fatal("We couldn't drop root group privileges");
+	env = getenv("RESTRICT_GID_FIRST");
+	if ((gid != 0 || (env != NULL && atoi(env) != 0)) && uid != 0) {
+		if (getgid() == 0 || getegid() == 0 || setgid(0) == 0) {
+			if (gid == 0)
+				i_fatal("GID 0 isn't permitted");
+			i_fatal("We couldn't drop root group privileges "
+				"(wanted=%s, gid=%s, egid=%s)", dec2str(gid),
+				dec2str(getgid()), dec2str(getegid()));
+		}
 	}
 
 	/* clear the environment, so we don't fail if we get back here */
