@@ -5,7 +5,8 @@
    restrict_access_by_env() */
 void restrict_access_set_env(const char *user, uid_t uid, gid_t gid,
 			     const char *chroot_dir,
-			     gid_t first_valid_gid, gid_t last_valid_gid);
+			     gid_t first_valid_gid, gid_t last_valid_gid,
+			     const char *extra_groups);
 
 /* chroot, setuid() and setgid() based on environment variables.
    If disallow_roots is TRUE, we'll kill ourself if we didn't have the
