@@ -220,7 +220,7 @@ int main(int argc __attr_unused__, char *argv[], char *envp[])
 {
 	const char *name, *group_name;
 	struct ip_addr ip;
-	int i, fd = -1, master_fd = -1;
+	int i, fd = -1, master_fd = -1, ssl = FALSE;
 
 	is_inetd = getenv("DOVECOT_MASTER") == NULL;
 
@@ -268,6 +268,7 @@ int main(int argc __attr_unused__, char *argv[], char *envp[])
 				fd = ssl_proxy_new(fd, &ip);
 				if (fd == -1)
 					i_fatal("SSL initialization failed");
+				ssl = TRUE;
 			} else if (strncmp(argv[i], "--group=", 8) != 0)
 				i_fatal("Unknown parameter: %s", argv[i]);
 		}
@@ -277,7 +278,7 @@ int main(int argc __attr_unused__, char *argv[], char *envp[])
 	}
 
 	if (fd != -1)
-		(void)client_create(fd, &ip, TRUE);
+		(void)client_create(fd, &ip, ssl);
 
 	io_loop_run(ioloop);
 	main_deinit();
