@@ -326,6 +326,8 @@ static pid_t create_auth_process(struct auth_process_group *group)
 		env_put("USE_CYRUS_SASL=1");
 	if (group->set->verbose)
 		env_put("VERBOSE=1");
+	if (group->set->debug)
+		env_put("VERBOSE_DEBUG=1");
 
 	restrict_process_size(group->set->process_size, (unsigned int)-1);
 
