@@ -391,7 +391,7 @@ static void login_process_init_env(struct login_group *group, pid_t pid)
 	restrict_access_set_env(group->set->user,
 				group->set->uid, set->login_gid,
 				set->login_chroot ? set->login_dir : NULL,
-				0, 0);
+				0, 0, NULL);
 
 	env_put("DOVECOT_MASTER=1");
 
