@@ -108,7 +108,7 @@ struct auth_settings {
 	const char *username_chars;
 	const char *anonymous_username;
 
-	int use_cyrus_sasl, verbose;
+	int use_cyrus_sasl, verbose, debug;
 
 	unsigned int count;
 	unsigned int process_size;
