@@ -33,10 +33,12 @@ int cmd_capa(struct pop3_client *client, const char *args __attr_unused__)
 
 		str_append(str, "SASL");
 		for (i = 0; i < AUTH_MECH_COUNT; i++) {
+			/* kludgy: don't advertise plaintext mechanisms
+			   because we don't support initial response in
+			   SASL, causing AUTH PLAIN to break if it's used. */
 			if ((auth_mechs & auth_mech_desc[i].mech) &&
 			    auth_mech_desc[i].name != NULL &&
-			    (client->tls || !auth_mech_desc[i].plaintext ||
-			     !disable_plaintext_auth)) {
+			    !auth_mech_desc[i].plaintext) {
 				str_append_c(str, ' ');
 				str_append(str, auth_mech_desc[i].name);
 			}
@@ -97,7 +99,8 @@ static void master_callback(struct client *_client, int success)
 	} else {
 		reason = t_strconcat("Internal login failure: ",
 				     client->common.virtual_user, NULL);
-		client_send_line(client, "* BYE Internal login failure.");
+		client_send_line(client, "* BYE Internal login failure. "
+				 "Error report written to server log.");
 	}
 
 	client_destroy(client, reason);
@@ -155,7 +158,7 @@ static void login_callback(struct auth_request *request,
 
 int cmd_user(struct pop3_client *client, const char *args)
 {
-	if (!client->tls && disable_plaintext_auth) {
+	if (!client->secured && disable_plaintext_auth) {
 		client_send_line(client,
 				 "-ERR Plaintext authentication disabled.");
 		return TRUE;
@@ -282,7 +285,7 @@ int cmd_auth(struct pop3_client *client, const char *args)
 		return TRUE;
 	}
 
-	if (!client->tls && mech->plaintext && disable_plaintext_auth) {
+	if (!client->secured && mech->plaintext && disable_plaintext_auth) {
 		client_send_line(client,
 				 "-ERR Plaintext authentication disabled.");
 		return TRUE;
