@@ -126,7 +126,10 @@ struct client *client_create(int hin, int hout, struct mail_storage *storage)
 
 	client->mailbox = storage->open_mailbox(storage, "INBOX", FALSE, FALSE);
 	if (client->mailbox == NULL) {
+		i_error("Couldn't open INBOX: %s",
+			storage->get_last_error(storage, NULL));
 		client_send_line(client, "-ERR No INBOX for user.");
+		client_destroy(client);
 		return NULL;
 	}
 
