@@ -261,6 +261,11 @@ static void stream_send_escaped(struct ostream *output, struct istream *input,
 				add = '.';
 				i++;
 				break;
+			} else if (data[i] == '\0' &&
+				   (client_workarounds &
+				    WORKAROUND_OUTLOOK_POP3_NO_NULS) != 0) {
+				add = '\x80';
+				break;
 			}
 		}
 
@@ -271,6 +276,11 @@ static void stream_send_escaped(struct ostream *output, struct istream *input,
 			if (o_stream_send(output, &add, 1) < 0)
 				return;
 			last = add;
+			if (client_workarounds &
+			    WORKAROUND_OUTLOOK_POP3_NO_NULS) {
+				if (i < size && data[i] == '\0')
+					i++;
+			}
 		} else {
 			last = data[i-1];
 		}
@@ -282,6 +292,9 @@ static void stream_send_escaped(struct ostream *output, struct istream *input,
 		/* didn't end with CRLF */
 		(void)o_stream_send(output, "\r\n", 2);
 	}
+
+	if (in_header)
+		(void)o_stream_send(output, "\r\n", 2);
 }
 
 static void fetch(struct client *client, unsigned int msgnum,
