@@ -20,6 +20,8 @@ struct ioloop *ioloop;
 static struct module *modules;
 static char log_prefix[128]; /* syslog() needs this to be permanent */
 
+void rawlog_open(int *hin, int *hout);
+
 static void sig_quit(int signo __attr_unused__)
 {
 	io_loop_stop(ioloop);
@@ -63,12 +65,16 @@ static int main_init(void)
 {
 	struct mail_storage *storage;
 	const char *mail;
+	int hin, hout;
 
 	lib_init_signals(sig_quit);
 
 	if (getenv("USER") == NULL)
 		i_fatal("USER environment missing");
 
+	hin = 0; hout = 1;
+	rawlog_open(&hin, &hout);
+
         mail_storage_init();
 	mail_storage_register_all();
 	clients_init();
@@ -100,7 +106,7 @@ static int main_init(void)
 		}
 	}
 
-	return client_create(0, 1, storage) != NULL;
+	return client_create(hin, hout, storage) != NULL;
 }
 
 static void main_deinit(void)
